import asyncio
import sys
import signal
import psutil
import os
import traceback

if sys.platform == 'linux':
    sys.path.append(os.getcwd() + "/funciones")
else:
    sys.path.append(os.getcwd() + "\\funciones")

from dotenv import load_dotenv

from funciones.principales import monitorear_chat, monitorear_mensajes, enviar_pendientes_a_usuarios, \
    obtener_pendientes_y_enviar, obtener_mensajes_para_enviar

from funciones.asincrono import abrir_navegador, apagar

from funciones.utiles import parsear_bytes
from bd.bd import bootstrap_bd

import selenium.common.exceptions as seleniumexc

from datetime import datetime, timedelta

from loguru import logger

logger.add("atwsp.log", enqueue=True)

load_dotenv()

MINUTOS_LOGUEO = 2

import web_pdb


async def main():
    # web_pdb.set_trace()
    print("iniciando main")
    # Inicializamos la base de datos, creando las tablas si no existen
    try:
        await bootstrap_bd()
    except:
        print("falla ini")
    # Conectamos con el driver de selenium, abriendo asi el navegador con whatsapp
    driver = abrir_navegador()
    ultimo_logeo = await obtener_fechahora_actual()  # Inicializamos el horario de logeo
    loop = asyncio.get_running_loop()
    print("ejecuto loop")
    # CODIGO DE SEÑALES COMENTADO PORQUE EN WINDOWS NO FUNCIONA
    # # Definimos las señales ante las cuales el programa debe apagarse
    # signals = (signal.SIGHUP, signal.SIGTERM, signal.SIGINT)
    #
    # for s in signals:  # Agregamos las señales al loop
    #     # Nuestro handler para las señales definidas arriba, es la corutina 'apagar', la cual se encargara de
    #     # finalizar la sesion del webdriver (NO LA VENTANA, LA SESION), y de cancelar todas las corutinas pendientes
    #     # liberando asi los recursos que estaban ocupando.
    #     loop.add_signal_handler(
    #         s, lambda s=s: asyncio.create_task(apagar(loop, driver)))

    # print("antes del while")
    while True:
        # Con este try verificamos si el webdriver se cerro, en caso de estarlo, enviamos una señal para
        # que se cancelen todas las tareas, liberando asi los recursos que estan ocupando
        try:

            _ = driver.window_handles
        except (seleniumexc.InvalidSessionIdException,
                seleniumexc.WebDriverException):
            print("error")
            logger.warning("Finalizando el programa, se cerro el webdriver")
            # os.kill(os.getpid(), signal.SIGTERM)
            await apagar(loop, driver)
            quit()

        respuestas = list()
        try:
            await asyncio.sleep(1)

            rta_monitor = await monitorear_chat(driver)
            respuestas.append(rta_monitor)

            # await asyncio.sleep(1)

            rta_obtener_para_enviar = await obtener_mensajes_para_enviar()
            respuestas.append(rta_obtener_para_enviar)

            # await asyncio.sleep(1)

            rta_enviar_atapis = await obtener_pendientes_y_enviar()
            respuestas.append(rta_enviar_atapis)

            # await asyncio.sleep(1)
            # rta_monitor = await monitorear_chat(driver)
            # respuestas.append(rta_monitor)

            rta_monitor_msjs = await monitorear_mensajes(driver)
            respuestas.append(rta_monitor_msjs)

            # await asyncio.sleep(1)
            # rta_monitor = await monitorear_chat(driver)
            # respuestas.append(rta_monitor)

            rta_enviar_usuarios = await enviar_pendientes_a_usuarios(driver)
            respuestas.append(rta_enviar_usuarios)

            # rta_monitor = await monitorear_chat(driver)
            # respuestas.append(rta_monitor)
        except Exception as error:
            print(f"{error}")

        for rta in respuestas:
            # Si alguna de las corutinas levanto una excepcion o devolvio respuesta False
            if isinstance(rta, Exception):
                logger.warning("Una de las corutinas levanto excepcion: {}", rta)
                try:
                    raise rta
                except:
                    print(traceback.format_exc())

            if isinstance(rta, dict) and rta['Estado'] is False:
                logger.warning("Una de las corutinas devolvio estado False: {}", rta)

        # Logearemos el hardware cada tantos minutos
        if await debe_loguear_recursos(ultimo_logeo, MINUTOS_LOGUEO):
            ram = await obtener_estado_memoria_virtual()

            await loguear_recursos(ram)

            ultimo_logeo = await obtener_fechahora_actual()


async def debe_loguear_recursos(ultimo_logeo, minutos):
    return await obtener_fechahora_actual() - timedelta(minutes=minutos) >= ultimo_logeo


async def obtener_fechahora_actual():
    return datetime.now()


async def loguear_recursos(ram):
    logger.info("Uso del CPU: {}% | RAM libre: {} | RAM usada: {}",
                psutil.cpu_percent(), parsear_bytes(ram.available), parsear_bytes(ram.used))


async def obtener_estado_memoria_virtual():
    return psutil.virtual_memory()


if __name__ == "__main__":
    asyncio.run(main(), debug=False)
